@@ -11,7 +11,7 @@ use AutoLoader;
 use vars qw($AUTOLOAD);
 use Carp;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 sub AUTOLOAD {
     my ($self, @args) = @_;
@@ -146,7 +146,66 @@ sub title_nr {
     $self->_vmg->title_nr($self->{titleid});
 }
 
-=head2 extract_chapter($chapter, $out)
+=head2 extract($out, $progress)
+
+Copy video data for the wall title into $out where:
+
+$out is either a file path or an open file handle
+
+$progress, if set, is a function reference accepting four args:
+the current cells count read and total cells count to read, the current
+block read for the cell, the total block for current cell.
+This allow you to show the current read progress.
+
+=cut
+
+sub extract {
+    my ($self, $outf, $progress) = @_;
+
+    $progress ||= sub {};
+
+    my $out;
+    if (ref $outf eq 'GLOB') {
+        $out = $outf;
+    } else {
+        open ($out, '>', $outf) or return;
+    }
+
+    my $pgc = $self->pgc(
+        $self->pgc_id($self->_vmg->title_ttn($self->{titleid}), 1) || 0
+    ) or return;
+
+    my $dvdfile = DVD::Read::Dvd::File->new(
+        $self->_dvd->{dvd},
+        $self->title_nr,
+        'VOB',
+    ) or return;
+
+    my $count = 0;
+    my $cells_count = $pgc->cells_count;
+    foreach my $cellid (
+        map { $pgc->cell_number($_) }
+        (1 .. $cells_count)) {
+        my $cell = $pgc->cell($cellid);
+        foreach my $sector ($cell->first_sector .. $cell->last_sector) {
+            $progress->($cellid -1, $cells_count, $sector - $cell->first_sector,
+            $cell->last_sector - $cell->first_sector +1); 
+            my ($co, $data) = $dvdfile->readblock($sector, 1);
+            $co or return;
+            $count += $co;
+            print $out $data;
+        }
+        $progress->($cellid, $cells_count, 0, 0);
+    }
+
+    if (ref $outf ne 'GLOB') {
+        close($out);
+    }
+
+    return $count;
+}
+
+=head2 extract_chapter($chapter, $out, $progress)
 
 Copy video data for chapter $chapter into $out where:
 
@@ -155,10 +214,18 @@ $out is either a file path or an open file handle
 $chapter either the chapter number to extract or an array of chapters
 to extract.
 
+$progress, if set, is a function reference acception four args:
+the current chapter count read, the count of chapter to read,
+the current block count read in for current chapter, the count
+of block to read for the current chapter.
+This allow you to give a progression status.
+
 =cut
 
 sub extract_chapter {
-    my ($self, $chapter, $outf) = @_;
+    my ($self, $chapter, $outf, $progress) = @_;
+
+    $progress ||= sub {};
 
     my $out;
     if (ref $outf eq 'GLOB') {
@@ -178,14 +245,21 @@ sub extract_chapter {
     ) or return;
 
     my $count = 0;
+    my $chapter_count = 0;
     foreach my $ch (@chapters) {
+
         defined(my $fs = $self->chapter_first_sector($ch)) or return;
         defined(my $ls = $self->chapter_last_sector($ch))  or return;
         foreach($fs .. $ls) {
+            $progress->($chapter_count, scalar(@chapters),
+                        $_ - $fs, $ls - $fs);
             my ($co, $data) = $dvdfile->readblock($_, 1);
             $count += $co;
             print $out $data;
         }
+        $chapter_count++;
+        #finalyze
+        $progress->($chapter_count, scalar(@chapters), 0, 0);
     }
 
     if (ref $outf ne 'GLOB') {
